/* -------------------------------------------------------------------------
         EXTERN.H - This file must be included in all C extensions
--------------------------------------------------------------------------*/

typedef unsigned int		WORD;
typedef unsigned char		BYTE;
typedef unsigned long		DWORD;
typedef int			BOOLEAN;

typedef char			*STRING;
typedef BYTE 			*PTR;
typedef PTR 			*HANDLE;

typedef int 			(*RTN)();

#define TRUE			1
#define FALSE			0

/*
** Routines of the following types may be used
*/
#define FUNCTION		2		/* returns a value using ReturnValue */
#define HANDLER			1		/* doesn't return a value */

/*
** HyperPAD requires a return code from the C routine indicating what to do
**    with the message.
**
**	PASS_TO_HPAD	-	pass the message directly to HyperPAD,
**				bypassing any objects which may normally
**				get called
**	PASS		-	pass the message on to the next object
**	STOP		-	don't pass the message on at all (useful for
**				errors and things)
**	ERROR		-	stops execution of all pending handlers
*/
#define PASS_TO_HPAD		2
#define PASS			0
#define STOP			1
#define ERROR			3

/*
** This defines the table structure for declaring user defined routines.
** One of these must exist in every external resource
*/
typedef struct {
	STRING Name;		/* 4 byte pointer to name of this procedure */
	RTN EntryPoint;		/* 4 byte pointer to this function */
	WORD MsgNum;		/* 2 byte relocated message number */
	WORD MsgType;		/* 2 byte - function or handler */
} POOL;
typedef POOL			*POOLPTR;

typedef struct PacketStruct {
	PTR Screen;		/* screen */
	BYTE bpl;    		/* bytes/line */
	BYTE Height; 		/* height of screen */
	BYTE Width;  		/* width of screen in characters */
	BYTE Flag;		/* flag with a bunch of booleans */
	RTN SetMode;		/* tells if the mode is supported */
	RTN CursorOFF;		/* makes the cursor invisible */
	RTN CursorThin;		/* makes the cursor thin */
	RTN CursorFat;		/* makes the cursor full character height */
	RTN ModeAvail;		/* is this mode available? */
} PACKET, *PACKETPTR;

#pragma pack(1)
typedef struct {
	BYTE Event;			/* event */
	BYTE ShiftStatus;		/* shift status at the time of the event */
	BYTE MouseX;			/* mouse x coordinate at the time of the event */
	BYTE MouseY;			/* mouse y coordinate at the time of the event */
	BYTE Buttons;			/* mouse buttons at the time of this event */
	WORD KeyWord;			/* the keyword (if this is a keypress event) */
	BYTE AsciiCode;			/* the ASCII code (if this is a keypress event) */
	BYTE ScanCode;			/* the scan code (if this is a keypress event) */
} MSG, *MSGPTR, **MSGHANDLE;

typedef struct {
	BYTE flags;			/* 0: flags */
	DWORD CmdLineBits;		/* 1: command line switches */
	char program_parms[128];	/* 5: original command line parameters */
	char program_name[80];		/* 133: name of original program */
	char spawn_name[80];		/* 213: name of program to spawn */
	char spawn_parms[128];		/* 293: parameters to a spawned program */
	char saved_pad[80];		/* 421: name of the pad to execute the second time around */
	BYTE return_code;		/* 501: return code from the sub-process */
	BYTE CardType[2];		/* 502: card types */
	BYTE MonitorType[2];		/* 504: monitor types */
	BYTE CardMem[2];		/* 506: video card memories */
	char CardName[30];		/* 508: the video card name */
	PACKETPTR *pVideoModes;		/* 538: pointer to the video modes */
	int (pascal far *StartRtn)();	/* 542: routine to start macro playback */
	int (pascal far *StopRtn)();	/* 546: routine to stop playback */
	int (pascal far *SaveKey)(WORD);/* 550: routine to record a keystroke */
	char OriginalDir[64];		/* 554: original directory (when running another program) */
	WORD EnvSeg;			/* 618: segment of the copy of the environment */
	DWORD SerialNo;			/* 620: Serial Number of this Exec used in runtimes */
	WORD EnvSegSz;			/* 624: number of bytes on environment segment */
	RTN SetNorm; 			/* 626: routine to set the video mode back to normal on exit */
	DWORD Reserved;			/* 630: */
	PTR TradeNamePtr;		/* 634: pointer to the name of the program */
} SHARED,*SHAREDPTR;
#pragma pack()

#define EVENT_IDLE	     	1	/* nothing is in the buffer */
#define EVENT_LEFTBUTTONDOWN 	2	/* left mouse button down */
#define EVENT_RIGHTBUTTONDOWN	3	/* right mouse button down */
#define EVENT_LEFTBUTTONUP   	4	/* left mouse button up */
#define EVENT_RIGHTBUTTONUP  	5	/* right mouse button up */
#define EVENT_KEYPRESS	     	6	/* scancode ready in the buffer */
#define EVENT_DOUBLECLICK    	7	/* left button clicked twice */

#define NULL			0L
#define deref(hdl)		(*(hdl))	/* macro to de-reference a handle: same as function htos() */

/* function prototypes to make sure these are declared this way */
extern pascal far WhenLoaded();
extern pascal far WhenUnLoaded();
extern POOL pascal Pool[];

/*
** The following are function declarations for HPAD's callbacks
*/

extern HANDLE pascal far NewHandle(WORD);	/* SizeInBytes */
extern pascal far FreeHandle(HANDLE);		/* HandleToFree */
extern DWORD pascal far HandleSize(HANDLE);	/* HandleToGetSizeOf */
extern PTR pascal far LockHandle(HANDLE);	/* HandleToLock */
extern pascal far UnLockHandle(HANDLE);		/* HandleToUnLock */
extern pascal far ReAllocHandle(HANDLE,WORD);	/* HandleToResize,NewSize */

extern PTR pascal far NewPtr(WORD);		/* SizeInBytes */
extern pascal far FreePtr(PTR);  		/* PtrToFree */
extern DWORD pascal far PtrSize(PTR);		/* PtrToGetSizeOf */

extern HANDLE pascal far itoh(int);		/* IntegerToConvert */
extern HANDLE pascal far ltoh(long);		/* LongToVonvert */
extern HANDLE pascal far stoh(STRING);		/* StringToConvert */
extern HANDLE pascal far ftoh(double);		/* DoubleToConvert */
extern HANDLE pascal far btoh(int);		/* Bool */
extern int pascal far htoi(HANDLE);		/* hText */
extern long pascal far htol(HANDLE);		/* hText */
extern STRING pascal far htos(HANDLE);		/* HandleToConvert */
extern int pascal far htob(HANDLE);		/* hText */
extern double htof(HANDLE);		/* HandleToDoubleText */

extern HANDLE pascal far GetFieldNum(int,WORD);	/* IsPage,FieldNum */
extern HANDLE pascal far GetFieldName(int,HANDLE);	/* IsPage,hFieldName */
extern HANDLE pascal far GetFieldID(int,WORD);	/* IsPage,FieldID */
extern pascal far SetFieldNum(int,WORD,HANDLE);	/* IsPage,FieldNum,hContent */
extern pascal far SetFieldName(int,HANDLE,HANDLE);	/* IsPage,hFieldName,hContent */
extern pascal far SetFieldID(int,WORD,HANDLE);	/* IsPage,FieldID,hContent */

extern pascal far Do(HANDLE);			/* hText */
extern SendPageMsg(HANDLE,int,...);	/* hMsg,NumArgs,... */
extern SendHPMsg(HANDLE,int,...);	/* hMsg,NumArgs,... */

extern pascal far SetGlobal(HANDLE,HANDLE);   	/* hGlobalName,hValue */
extern HANDLE pascal far GetGlobal(HANDLE);	/* hGlobalName */
extern HANDLE pascal far GetGlobals();

extern pascal far PeekEvent(MSGPTR);	/* pMsg */
extern pascal far GetEvent(MSGPTR);	/* pMsg */
extern pascal far UnGetEvent(MSGPTR);	/* pMsg */

extern pascal far MakeWin(int,int,int,int,int,PACKETPTR);	/* x1,y1,x2,y2,attr,Screen */
extern pascal far RemWin();
extern pascal far PutStr(int,int,char far *,PACKETPTR);		/* x,y,str,Screen */
extern pascal far HighLine(int,int,int,int,PACKETPTR);		/* x1,x2,y,attr,Screen */
extern pascal far PutCh(int,int,int,PACKETPTR);		/* x,y,ch,Screen */
extern pascal far PutAttr(int,int,int,PACKETPTR);	/* x,y,attr,Screen */
extern pascal far SetScreen(int,int,int,int,int,PTR);	/* IsPage,x1,y1,x2,y2,pData */
extern HANDLE pascal far GetScreen(int,int,int,int,int);	/* IsPage,x1,y1,x2,y2 */
extern PACKETPTR pascal far ScreenPtr(int);	/* IsPage */
extern pascal far CopyPart(int,int,int,int,PACKETPTR,PACKETPTR);	/* x1,y1,x2,y2,Source,Target */
extern RefreshPart(int,int,int,int);	/* x1,y1,x2,y2 */
extern Refresh();
extern pascal far ShowMouse();
extern pascal far HideMouse();

extern pascal far SetKeyHook(int (_loadds *)(WORD));

extern pascal far ReleaseHPAD();
extern pascal far RestoreHPAD();

extern DWORD pascal far GetTime();
extern SHAREDPTR pascal far GetSharedArea();
extern pascal far ReturnValue(HANDLE);		/* VarHandle */

extern WORD pascal far TranslateKey(PTR);	// KeyString

/*
** regular C functions that are HyperPAD callbacks
*/
int cdecl sprintf(char *, const char *, ...);
int cdecl printf(const char *,...);

/*
** Math Functions:
**
** The following math function are defined in HyperPAD. These are the
** only math function that work correctly from within extensions (others
** are untested).
*/
extern double acos(double);
extern double cos(double);
extern double sin(double);
extern double asin(double);
extern double tan(double);
extern double atan(double);
extern double exp(double);
extern double pow(double,double);
extern double log(double);
extern double fabs(double);
extern double fmod(double,double);
extern double floor(double);
extern double sqrt(double);
